{ Main view, where most of the application logic takes place.

  Feel free to use this code as a starting point for your own projects.
  This template code is in public domain, unlike most other CGE code which
  is covered by BSD or LGPL (see https://castle-engine.io/license). }
unit GameViewMain;

interface

uses Classes,
  CastleVectors, CastleComponentSerialize,CastleScene,X3DNodes,
  CastleUIControls, CastleControls, CastleKeysMouse,CastleViewport;

type

  TMyClass = class(TCastleAbstractPrimitive)
    private
      FGeometry: TIndexedTriangleSetNode;
      FCoordinate: TCoordinateNode;
      FTextureCoordinate: TTextureCoordinateNode;
    public
     constructor Create(AOwner: TComponent); override;
     procedure ReLoad;
  end;

  { Main view, where most of the application logic takes place. }
  TViewMain = class(TCastleView)
  private
   MyObject:TMyClass;
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    LabelFps: TCastleLabel;
    Viewport1:TCastleViewport;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Stop; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
    function Press(const Event: TInputPressRelease): Boolean; override;
  end;

var
  ViewMain: TViewMain;

implementation

uses SysUtils;


constructor TMyClass.Create(AOwner: TComponent);
begin
  inherited;
  Self.Texture:='castle-data:/test_texture.png';
  Self.Material:=pmUnlit;
  FGeometry:=TIndexedTriangleSetNode.Create;
  FCoordinate:=TCoordinateNode.Create;
  FTextureCoordinate:=TTextureCoordinateNode.Create;
  FGeometry.Coord:=FCoordinate;
  ShapeNode.Geometry:=FGeometry;
end;

procedure TMyClass.Reload;
begin
  FCoordinate.SetPoint([
      Vector3(0  ,   0, 0),
      Vector3(100,   0, 0),
      Vector3(100, 100, 0),
      Vector3(0  , 100, 0),
      Vector3(0  ,   0, 0),
      Vector3(100, 100, 0)
    ]);
  FTextureCoordinate.SetPoint([
    Vector2(0, 0),
    Vector2(1, 0),
    Vector2(1, 1),
    Vector2(0, 1),
    Vector2(0, 0),
    Vector2(1, 1)
  ]);
  FGeometry.SetIndex([0, 1, 2, 3, 4, 5]);
  FGeometry.TexCoord:= FTextureCoordinate;

end;

{ TViewMain ----------------------------------------------------------------- }

constructor TViewMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gameviewmain.castle-user-interface';
end;

procedure TViewMain.Start;
begin
  inherited;
  MyObject:=TMyClass.Create(Self);
  Viewport1.Items.Add(MyObject);
  MyObject.ReLoad;

end;
procedure TViewMain.Stop;
begin
  MyObject.Free;
end;

procedure TViewMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
begin
  inherited;
  { This virtual method is executed every frame (many times per second). }
  Assert(LabelFps <> nil, 'If you remove LabelFps from the design, remember to remove also the assignment "LabelFps.Caption := ..." from code');
  LabelFps.Caption := 'FPS: ' + Container.Fps.ToString;
end;

function TViewMain.Press(const Event: TInputPressRelease): Boolean;
begin
  Result := inherited;
  if Result then Exit; // allow the ancestor to handle keys

  { This virtual method is executed when user presses
    a key, a mouse button, or touches a touch-screen.

    Note that each UI control has also events like OnPress and OnClick.
    These events can be used to handle the "press", if it should do something
    specific when used in that UI control.
    The TViewMain.Press method should be used to handle keys
    not handled in children controls.
  }

  // Use this to handle keys:
  {
  if Event.IsKey(keyXxx) then
  begin
    // DoSomething;
    Exit(true); // key was handled
  end;
  }
end;

end.
